'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var compositionApi = require('@vue/composition-api');
var axios = _interopDefault(require('axios'));
var core = require('@vue-composable/core');

function useAxios(configUrlThrowException, configThrowException, throwException = false) {
    const config = !core.isString(configUrlThrowException) && !core.isBoolean(configUrlThrowException)
        ? configUrlThrowException
        : core.isObject(configThrowException)
            ? configThrowException
            : undefined;
    throwException = core.isBoolean(configUrlThrowException)
        ? configUrlThrowException
        : core.isBoolean(configThrowException)
            ? configThrowException
            : throwException;
    const axiosClient = axios.create(config);
    const client = compositionApi.computed(() => axiosClient);
    const isCancelled = compositionApi.ref(false);
    const cancelledMessage = compositionApi.ref(null);
    let cancelToken = undefined;
    const cancel = (message) => {
        if (!cancelToken) {
            /* istanbul ignore else */
            {
                return;
            }
        }
        cancelToken.cancel(message);
        isCancelled.value = true;
        cancelledMessage.value = message;
    };
    const use = core.usePromise(async (request) => {
        cancelToken = axios.CancelToken.source();
        isCancelled.value = false;
        cancelledMessage.value = null;
        const opts = core.isString(request) ? { url: request } : request;
        return axiosClient.request({
            cancelToken: cancelToken.token,
            ...opts
        });
    }, throwException);
    const data = compositionApi.computed(() => (use.result.value && use.result.value.data) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.data) ||
        null);
    const status = compositionApi.computed(() => (use.result.value && use.result.value.status) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.status) ||
        null);
    const statusText = compositionApi.computed(() => (use.result.value && use.result.value.statusText) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.statusText) ||
        null);
    // if url provided in the config, execute it straight away
    // NOTE: `false` is passed to the `exec` to prevent the exception to be thrown
    if (typeof configUrlThrowException === "string") {
        use.exec({ ...config, url: configUrlThrowException }, false);
    }
    else if (config && config.url) {
        use.exec(config, false);
    }
    return {
        ...use,
        client,
        data,
        status,
        statusText,
        cancel,
        isCancelled,
        cancelledMessage
    };
}

exports.useAxios = useAxios;
